@@ -23,13 +23,13 @@ from hyperliquid.utils.signing import (
     order_wires_to_order_action,
     sign_agent,
     sign_approve_builder_fee,
+    sign_convert_to_multi_sig_user_action,
     sign_l1_action,
+    sign_multi_sig_action,
     sign_spot_transfer_action,
     sign_usd_class_transfer_action,
     sign_usd_transfer_action,
     sign_withdraw_from_bridge_action,
-    sign_convert_to_multi_sig_user_action,
-    sign_convert_to_multi_sig_signer_action,
 )
 from hyperliquid.utils.types import Any, BuilderInfo, Cloid, List, Meta, Optional, SpotMeta, Tuple
 
@@ -75,13 +75,14 @@ class Exchange(API):
             # Get midprice
             px = float(self.info.all_mids()[coin])
 
+        asset = self.info.coin_to_asset[coin]
         # spot assets start at 10000
-        is_spot = self.info.coin_to_asset[coin] >= 10_000
+        is_spot = asset >= 10_000
 
         # Calculate Slippage
         px *= (1 + slippage) if is_buy else (1 - slippage)
         # We round px to 5 significant figures and 6 decimals for perps, 8 decimals for spot
-        return round(float(f"{px:.5g}"), 6 if not is_spot else 8)
+        return round(float(f"{px:.5g}"), (6 if not is_spot else 8) - self.info.asset_to_sz_decimals[asset])
 
     def order(
         self,
@@ -112,6 +113,8 @@ class Exchange(API):
         ]
         timestamp = get_timestamp_ms()
 
+        if builder:
+            builder["b"] = builder["b"].lower()
         order_action = order_wires_to_order_action(order_wires, builder)
 
         signature = sign_l1_action(
@@ -208,7 +211,7 @@ class Exchange(API):
         cloid: Optional[Cloid] = None,
         builder: Optional[BuilderInfo] = None,
     ) -> Any:
-        address = self.wallet.address
+        address: str = self.wallet.address
         if self.account_address:
             address = self.account_address
         if self.vault_address:
@@ -422,30 +425,6 @@ class Exchange(API):
             timestamp,
         )
 
-    # Deprecated in favor of usd_class_transfer
-    def user_spot_transfer(self, usdc: float, to_perp: bool) -> Any:
-        usdc = int(round(usdc, 2) * 1e6)
-        timestamp = get_timestamp_ms()
-        spot_user_action = {
-            "type": "spotUser",
-            "classTransfer": {
-                "usdc": usdc,
-                "toPerp": to_perp,
-            },
-        }
-        signature = sign_l1_action(
-            self.wallet,
-            spot_user_action,
-            self.vault_address,
-            timestamp,
-            self.base_url == MAINNET_API_URL,
-        )
-        return self._post_action(
-            spot_user_action,
-            signature,
-            timestamp,
-        )
-
     def sub_account_transfer(self, sub_account_user: str, is_deposit: bool, usd: int) -> Any:
         timestamp = get_timestamp_ms()
         sub_account_transfer_action = {
@@ -467,6 +446,28 @@ class Exchange(API):
             timestamp,
         )
 
+    def sub_account_spot_transfer(self, sub_account_user: str, is_deposit: bool, token: str, amount: float) -> Any:
+        timestamp = get_timestamp_ms()
+        sub_account_transfer_action = {
+            "type": "subAccountSpotTransfer",
+            "subAccountUser": sub_account_user,
+            "isDeposit": is_deposit,
+            "token": token,
+            "amount": str(amount),
+        }
+        signature = sign_l1_action(
+            self.wallet,
+            sub_account_transfer_action,
+            None,
+            timestamp,
+            self.base_url == MAINNET_API_URL,
+        )
+        return self._post_action(
+            sub_account_transfer_action,
+            signature,
+            timestamp,
+        )
+
     def vault_usd_transfer(self, vault_address: str, is_deposit: bool, usd: int) -> Any:
         timestamp = get_timestamp_ms()
         vault_transfer_action = {
@@ -572,15 +573,125 @@ class Exchange(API):
             timestamp,
         )
 
-    def convert_to_multi_sig_signer(self, signer: str, multi_sig_user: str) -> Any:
+    def spot_deploy_register_token(
+        self, token_name: str, sz_decimals: int, wei_decimals: int, max_gas: int, full_name: str
+    ) -> Any:
         timestamp = get_timestamp_ms()
         action = {
-            "type": "convertToMultiSigUser",
-            "signer": signer,
-            "multi_sig_user": multi_sig_user,
-            "nonce": timestamp,
+            "type": "spotDeploy",
+            "registerToken2": {
+                "spec": {"name": token_name, "szDecimals": sz_decimals, "weiDecimals": wei_decimals},
+                "maxGas": max_gas,
+                "fullName": full_name,
+            },
         }
-        signature = sign_convert_to_multi_sig_signer_action(self.wallet, action, self.base_url == MAINNET_API_URL)
+        signature = sign_l1_action(
+            self.wallet,
+            action,
+            None,
+            timestamp,
+            self.base_url == MAINNET_API_URL,
+        )
+        return self._post_action(
+            action,
+            signature,
+            timestamp,
+        )
+
+    def spot_deploy_user_genesis(
+        self, token: int, user_and_wei: List[Tuple[str, str]], existing_token_and_wei: List[Tuple[int, str]]
+    ) -> Any:
+        timestamp = get_timestamp_ms()
+        action = {
+            "type": "spotDeploy",
+            "userGenesis": {
+                "token": token,
+                "userAndWei": [(user.lower(), wei) for (user, wei) in user_and_wei],
+                "existingTokenAndWei": existing_token_and_wei,
+            },
+        }
+        signature = sign_l1_action(
+            self.wallet,
+            action,
+            None,
+            timestamp,
+            self.base_url == MAINNET_API_URL,
+        )
+        return self._post_action(
+            action,
+            signature,
+            timestamp,
+        )
+
+    def spot_deploy_genesis(self, token: int, max_supply: str, no_hyperliquidity: bool) -> Any:
+        timestamp = get_timestamp_ms()
+        genesis = {
+            "token": token,
+            "maxSupply": max_supply,
+        }
+        if no_hyperliquidity:
+            genesis["noHyperliquidity"] = True
+        action = {
+            "type": "spotDeploy",
+            "genesis": genesis,
+        }
+        signature = sign_l1_action(
+            self.wallet,
+            action,
+            None,
+            timestamp,
+            self.base_url == MAINNET_API_URL,
+        )
+        return self._post_action(
+            action,
+            signature,
+            timestamp,
+        )
+
+    def spot_deploy_register_spot(self, base_token: int, quote_token: int) -> Any:
+        timestamp = get_timestamp_ms()
+        action = {
+            "type": "spotDeploy",
+            "registerSpot": {
+                "tokens": [base_token, quote_token],
+            },
+        }
+        signature = sign_l1_action(
+            self.wallet,
+            action,
+            None,
+            timestamp,
+            self.base_url == MAINNET_API_URL,
+        )
+        return self._post_action(
+            action,
+            signature,
+            timestamp,
+        )
+
+    def spot_deploy_register_hyperliquidity(
+        self, spot: int, start_px: float, order_sz: float, n_orders: int, n_seeded_levels: Optional[int]
+    ) -> Any:
+        timestamp = get_timestamp_ms()
+        register_hyperliquidity = {
+            "spot": spot,
+            "startPx": str(start_px),
+            "orderSz": str(order_sz),
+            "nOrders": n_orders,
+        }
+        if n_seeded_levels is not None:
+            register_hyperliquidity["nSeededLevels"] = n_seeded_levels
+        action = {
+            "type": "spotDeploy",
+            "registerHyperliquidity": register_hyperliquidity,
+        }
+        signature = sign_l1_action(
+            self.wallet,
+            action,
+            None,
+            timestamp,
+            self.base_url == MAINNET_API_URL,
+        )
         return self._post_action(
             action,
             signature,
@@ -588,22 +699,46 @@ class Exchange(API):
         )
 
     def multi_sig(self, multi_sig_user, inner_action, signatures, nonce, vault_address=None):
+        multi_sig_user = multi_sig_user.lower()
         multi_sig_action = {
             "type": "multiSig",
-            "user": multi_sig_user.lower(),
+            "signatureChainId": "0x66eee",
             "signatures": signatures,
-            "inner": inner_action,
+            "payload": {
+                "multiSigUser": multi_sig_user,
+                "outerSigner": self.wallet.address.lower(),
+                "action": inner_action,
+            },
         }
-        signature = sign_l1_action(
+        is_mainnet = self.base_url == MAINNET_API_URL
+        signature = sign_multi_sig_action(
             self.wallet,
             multi_sig_action,
+            is_mainnet,
             vault_address,
             nonce,
-            self.base_url == MAINNET_API_URL,
         )
-
         return self._post_action(
             multi_sig_action,
             signature,
             nonce,
         )
+
+    def use_big_blocks(self, enable: bool) -> Any:
+        timestamp = get_timestamp_ms()
+        action = {
+            "type": "evmUserModify",
+            "usingBigBlocks": enable,
+        }
+        signature = sign_l1_action(
+            self.wallet,
+            action,
+            None,
+            timestamp,
+            self.base_url == MAINNET_API_URL,
+        )
+        return self._post_action(
+            action,
+            signature,
+            timestamp,
+        )
